@@ -6,7 +6,13 @@ using namespace Rcpp;
 #ifndef __SURVIVAL_ANALYSIS__
 #define __SURVIVAL_ANALYSIS__
 
-NumericVector fsurvci(double surv, double sesurv, String ct, double z);
+NumericVector fsurvci(double surv, double sesurv, std::string ct, double z);
+
+DataFrame survQuantile(const NumericVector& time,
+                       const IntegerVector& event,
+                       const double cilevel,
+                       const std::string transform,
+                       const NumericVector& probs);
 
 DataFrame kmest(const DataFrame data,
                 const StringVector& rep,
@@ -36,7 +42,6 @@ DataFrame lrtest(const DataFrame data,
                  const double rho1,
                  const double rho2);
 
-
 DataFrame rmest(const DataFrame data,
                 const StringVector& rep,
                 const StringVector& stratum,
@@ -76,6 +81,10 @@ NumericVector f_score_1(int p, NumericVector par, void *ex);
 
 NumericMatrix f_info_1(int p, NumericVector par, void *ex);
 
+List f_der_eta_1(NumericVector eta, NumericVector sig, void *ex);
+
+List f_der_eta_tau_1(NumericVector eta, NumericVector sig, void *ex);
+
 NumericMatrix f_ressco_1(int p, NumericVector par, void *ex);
 
 NumericMatrix f_jj_1(int p, NumericVector par, void *ex);
@@ -101,11 +110,28 @@ List liferegcpp(const DataFrame data,
                 const std::string dist,
                 const bool robust,
                 const bool plci,
-                const double alpha);
+                const double alpha,
+                const int maxiter,
+                const double eps);
+
+NumericMatrix residuals_liferegcpp(const NumericVector& beta,
+                                   const NumericMatrix& vbeta,
+                                   DataFrame data,
+                                   const StringVector& stratum,
+                                   const std::string time,
+                                   const std::string time2,
+                                   const std::string event,
+                                   const StringVector& covariates,
+                                   const std::string weight,
+                                   const std::string offset,
+                                   const std::string id,
+                                   const std::string dist,
+                                   const std::string type,
+                                   const bool collapse,
+                                   const bool weighted);
 
 struct coxparams {
   int nused;
-  double delta;
   IntegerVector strata;
   NumericVector tstart;
   NumericVector tstop;
@@ -161,7 +187,9 @@ List phregcpp(const DataFrame data,
               const bool est_resid,
               const bool firth,
               const bool plci,
-              const double alpha);
+              const double alpha,
+              const int maxiter,
+              const double eps);
 
 DataFrame survfit_phregcpp(const int p,
                            const NumericVector& beta,
@@ -180,6 +208,8 @@ DataFrame survfit_phregcpp(const int p,
 
 List residuals_phregcpp(const int p,
                         const NumericVector& beta,
+                        const NumericMatrix& vbeta,
+                        const NumericVector& resmart,
                         DataFrame data,
                         const StringVector& stratum,
                         const std::string time,
@@ -190,6 +220,8 @@ List residuals_phregcpp(const int p,
                         const std::string offset,
                         const std::string id,
                         const std::string ties,
-                        const std::string type);
+                        const std::string type,
+                        const bool collapse,
+                        const bool weighted);
 
 #endif // __SURVIVAL_ANALYSIS__
