@@ -7,8 +7,8 @@ testthat::test_that("kmest: estimate and standard error", {
   df2 <- summary(survfit(Surv(time, status) ~ x, data=aml,
                          conf.type="none"))
 
-  testthat::expect_equal(df1$survival, df2$surv)
-  testthat::expect_equal(df1$stderr, df2$std.err)
+  testthat::expect_equal(df1$survival[df1$time > 0], df2$surv)
+  testthat::expect_equal(df1$stderr[df1$time > 0], df2$std.err)
 })
 
 
@@ -20,8 +20,8 @@ testthat::test_that("kmest: confidence interval", {
     df2 <- summary(survfit(Surv(time, status) ~ x, data=aml,
                            conf.type=conftype))
 
-    testthat::expect_equal(df1$lower, df2$lower)
-    testthat::expect_equal(df1$upper, df2$upper)
+    testthat::expect_equal(df1$lower[df1$time > 0], df2$lower)
+    testthat::expect_equal(df1$upper[df1$time > 0], df2$upper)
   }
 })
 
